@@ -1,21 +1,9 @@
-//
-// Copyright (C) 2016~2016 by CSSlayer
-// wengxt@gmail.com
-//
-// This library is free software; you can redistribute it and/or modify
-// it under the terms of the GNU Lesser General Public License as
-// published by the Free Software Foundation; either version 2.1 of the
-// License, or (at your option) any later version.
-//
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-// Lesser General Public License for more details.
-//
-// You should have received a copy of the GNU Lesser General Public
-// License along with this library; see the file COPYING. If not,
-// see <http://www.gnu.org/licenses/>.
-//
+/*
+ * SPDX-FileCopyrightText: 2016-2016 CSSlayer <wengxt@gmail.com>
+ *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ *
+ */
 #ifndef _FCITX_UTILS_DBUS_OBJECTVTABLE_H_
 #define _FCITX_UTILS_DBUS_OBJECTVTABLE_H_
 
@@ -38,6 +26,7 @@ class Bus;
 class ObjectVTablePrivate;
 
 typedef std::function<bool(Message)> ObjectMethod;
+typedef std::function<bool(Message, const ObjectMethod &)> ObjectMethodClosure;
 typedef std::function<void(Message &)> PropertyGetMethod;
 typedef std::function<bool(Message &)> PropertySetMethod;
 
@@ -70,6 +59,16 @@ public:
     const ObjectMethod &handler() const;
     ObjectVTableBase *vtable() const;
 
+    /**
+     * Set a closure function to call the handler with in it.
+     *
+     * This is useful when you want to do something before and after the dbus
+     * message delivery.
+     *
+     * @param wrapper wrapper function.
+     */
+    void setClosureFunction(ObjectMethodClosure closure);
+
 private:
     std::unique_ptr<ObjectVTableMethodPrivate> d_ptr;
     FCITX_DECLARE_PRIVATE(ObjectVTableMethod);
@@ -195,8 +194,8 @@ struct ReturnValueHelper<void> {
 class ObjectVTableSignalPrivate;
 class FCITXUTILS_EXPORT ObjectVTableSignal {
 public:
-    ObjectVTableSignal(ObjectVTableBase *vtable, const std::string &name,
-                       const std::string signature);
+    ObjectVTableSignal(ObjectVTableBase *vtable, std::string name,
+                       std::string signature);
     virtual ~ObjectVTableSignal();
 
     Message createSignal();
@@ -215,9 +214,9 @@ using PropertyOptions = Flags<PropertyOption>;
 class ObjectVTablePropertyPrivate;
 class FCITXUTILS_EXPORT ObjectVTableProperty {
 public:
-    ObjectVTableProperty(ObjectVTableBase *vtable, const std::string &name,
-                         const std::string signature,
-                         PropertyGetMethod getMethod, PropertyOptions options);
+    ObjectVTableProperty(ObjectVTableBase *vtable, std::string name,
+                         std::string signature, PropertyGetMethod getMethod,
+                         PropertyOptions options);
     virtual ~ObjectVTableProperty();
 
     const std::string &name() const;
@@ -236,9 +235,8 @@ protected:
 class FCITXUTILS_EXPORT ObjectVTableWritableProperty
     : public ObjectVTableProperty {
 public:
-    ObjectVTableWritableProperty(ObjectVTableBase *vtable,
-                                 const std::string &name,
-                                 const std::string signature,
+    ObjectVTableWritableProperty(ObjectVTableBase *vtable, std::string name,
+                                 std::string signature,
                                  PropertyGetMethod getMethod,
                                  PropertySetMethod setMethod,
                                  PropertyOptions options);
